@@ -32,7 +32,7 @@ virNetDevVlanParse(xmlNodePtr node, xmlXPathContextPtr ctxt, virNetDevVlanPtr de
 {
     int ret = -1;
     xmlNodePtr save = ctxt->node;
-    const char *trunk;
+    const char *trunk = NULL;
     xmlNodePtr *tagNodes = NULL;
     int nTags, ii;
 
@@ -103,6 +103,7 @@ virNetDevVlanParse(xmlNodePtr node, xmlXPathContextPtr ctxt, virNetDevVlanPtr de
 error:
     ctxt->node = save;
     VIR_FREE(tagNodes);
+    VIR_FREE(trunk);
     if (ret < 0)
         virNetDevVlanClear(def);
     return ret;
