@@ -36,7 +36,7 @@
 #define VIR_FROM_THIS VIR_FROM_SECRET
 
 VIR_ENUM_IMPL(virSecretUsageType, VIR_SECRET_USAGE_TYPE_LAST,
-              "none", "volume", "ceph")
+              "none", "volume", "ceph", "iscsi")
 
 void
 virSecretDefFree(virSecretDefPtr def)
@@ -57,6 +57,10 @@ virSecretDefFree(virSecretDefPtr def)
         VIR_FREE(def->usage.ceph);
         break;
 
+    case VIR_SECRET_USAGE_TYPE_ISCSI:
+        VIR_FREE(def->usage.target);
+        break;
+
     default:
         VIR_ERROR(_("unexpected secret usage type %d"), def->usage_type);
         break;
@@ -108,6 +112,15 @@ virSecretDefParseUsage(xmlXPathContextPtr ctxt,
         }
         break;
 
+    case VIR_SECRET_USAGE_TYPE_ISCSI:
+        def->usage.target = virXPathString("string(./usage/target)", ctxt);
+        if (!def->usage.target) {
+            virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
+                           _("iSCSI usage specified, but target is missing"));
+            return -1;
+        }
+        break;
+
     default:
         virReportError(VIR_ERR_INTERNAL_ERROR,
                        _("unexpected secret usage type %d"),
@@ -262,6 +275,13 @@ virSecretDefFormatUsage(virBufferPtr buf,
         }
         break;
 
+    case VIR_SECRET_USAGE_TYPE_ISCSI:
+        if (def->usage.target != NULL) {
+            virBufferEscapeString(buf, "    <target>%s</target>\n",
+                                  def->usage.target);
+        }
+        break;
+
     default:
         virReportError(VIR_ERR_INTERNAL_ERROR,
                        _("unexpected secret usage type %d"),
