@@ -27,6 +27,7 @@
 #include <config.h>
 
 #include "lxc_conf.h"
+#include "lxc_domain.h"
 #include "nodeinfo.h"
 #include "virerror.h"
 #include "virconf.h"
@@ -154,6 +155,13 @@ error:
     return NULL;
 }
 
+
+virDomainXMLConfPtr
+lxcDomainXMLConfInit(void)
+{
+    return virDomainXMLConfNew(&virLXCDriverPrivateDataCallbacks, NULL);
+}
+
 int lxcLoadDriverConfig(virLXCDriverPtr driver)
 {
     char *filename;
