@@ -188,6 +188,9 @@ struct _virQEMUDriver {
      */
     virCapsPtr caps;
 
+    /* Immutable pointer, Immutable object */
+    virDomainXMLConfPtr xmlconf;
+
     /* Immutable pointer, self-locking APIs */
     virQEMUCapsCachePtr qemuCapsCache;
 
@@ -298,5 +301,6 @@ void qemuSharedDiskEntryFree(void *payload, const void *name)
     ATTRIBUTE_NONNULL(1);
 
 int qemuDriverAllocateID(virQEMUDriverPtr driver);
+virDomainXMLConfPtr virQEMUDriverCreateXMLConf(void);
 
 #endif /* __QEMUD_CONF_H */
