@@ -54,7 +54,8 @@ int virPipeReadUntilEOF(int outfd, int errfd,
                         char **outbuf, char **errbuf);
 
 int virSetUIDGID(uid_t uid, gid_t gid);
-int virSetUIDGIDWithCaps(uid_t uid, gid_t gid, unsigned long long capBits);
+int virSetUIDGIDWithCaps(uid_t uid, gid_t gid, unsigned long long capBits,
+                         bool clearExistingCaps);
 
 int virFileReadLimFD(int fd, int maxlen, char **buf) ATTRIBUTE_RETURN_CHECK;
 
@@ -296,5 +297,24 @@ int virGetDeviceUnprivSGIO(const char *path,
                            int *unpriv_sgio);
 char * virGetUnprivSGIOSysfsPath(const char *path,
                                  const char *sysfs_dir);
+int virReadFCHost(const char *sysfs_prefix,
+                  int host,
+                  const char *entry,
+                  char **result)
+    ATTRIBUTE_NONNULL(3) ATTRIBUTE_NONNULL(4);
+
+int virIsCapableFCHost(const char *sysfs_prefix, int host);
+int virIsCapableVport(const char *sysfs_prefix, int host);
+
+enum {
+    VPORT_CREATE,
+    VPORT_DELETE,
+};
+
+int virManageVport(const int parent_host,
+                   const char *wwpn,
+                   const char *wwnn,
+                   int operation)
+    ATTRIBUTE_NONNULL(2) ATTRIBUTE_NONNULL(3);
 
 #endif /* __VIR_UTIL_H__ */
