@@ -803,7 +803,7 @@ mymain(void)
             QEMU_CAPS_NEC_USB_XHCI,
             QEMU_CAPS_NETDEV_BRIDGE,
             QEMU_CAPS_SCSI_LSI,
-            QEMU_CAPS_VIRTIO_SCSI_PCI,
+            QEMU_CAPS_VIRTIO_SCSI,
             QEMU_CAPS_BLOCKIO,
             QEMU_CAPS_VNC,
             QEMU_CAPS_DEVICE_QXL,
@@ -811,7 +811,8 @@ mymain(void)
             QEMU_CAPS_DEVICE_CIRRUS_VGA,
             QEMU_CAPS_DEVICE_VMWARE_SVGA,
             QEMU_CAPS_DEVICE_USB_SERIAL,
-            QEMU_CAPS_DEVICE_USB_NET);
+            QEMU_CAPS_DEVICE_USB_NET,
+            QEMU_CAPS_DTB);
     DO_TEST("qemu-1.2.0", 1002000, 0, 0,
             QEMU_CAPS_VNC_COLON,
             QEMU_CAPS_NO_REBOOT,
@@ -892,7 +893,7 @@ mymain(void)
             QEMU_CAPS_NEC_USB_XHCI,
             QEMU_CAPS_NETDEV_BRIDGE,
             QEMU_CAPS_SCSI_LSI,
-            QEMU_CAPS_VIRTIO_SCSI_PCI,
+            QEMU_CAPS_VIRTIO_SCSI,
             QEMU_CAPS_BLOCKIO,
             QEMU_CAPS_DISABLE_S3,
             QEMU_CAPS_DISABLE_S4,
@@ -910,7 +911,9 @@ mymain(void)
             QEMU_CAPS_DEVICE_VMWARE_SVGA,
             QEMU_CAPS_DEVICE_VIDEO_PRIMARY,
             QEMU_CAPS_DEVICE_USB_SERIAL,
-            QEMU_CAPS_DEVICE_USB_NET);
+            QEMU_CAPS_DEVICE_USB_NET,
+            QEMU_CAPS_DTB,
+            QEMU_CAPS_SCSI_MEGASAS);
     DO_TEST("qemu-kvm-1.2.0", 1002000, 1, 0,
             QEMU_CAPS_VNC_COLON,
             QEMU_CAPS_NO_REBOOT,
@@ -996,7 +999,7 @@ mymain(void)
             QEMU_CAPS_NEC_USB_XHCI,
             QEMU_CAPS_NETDEV_BRIDGE,
             QEMU_CAPS_SCSI_LSI,
-            QEMU_CAPS_VIRTIO_SCSI_PCI,
+            QEMU_CAPS_VIRTIO_SCSI,
             QEMU_CAPS_BLOCKIO,
             QEMU_CAPS_DISABLE_S3,
             QEMU_CAPS_DISABLE_S4,
@@ -1014,7 +1017,9 @@ mymain(void)
             QEMU_CAPS_DEVICE_VMWARE_SVGA,
             QEMU_CAPS_DEVICE_VIDEO_PRIMARY,
             QEMU_CAPS_DEVICE_USB_SERIAL,
-            QEMU_CAPS_DEVICE_USB_NET);
+            QEMU_CAPS_DEVICE_USB_NET,
+            QEMU_CAPS_DTB,
+            QEMU_CAPS_SCSI_MEGASAS);
 
     return ret == 0 ? EXIT_SUCCESS : EXIT_FAILURE;
 }
