@@ -44,6 +44,7 @@
 #define VIR_FROM_THIS VIR_FROM_NONE
 
 static virCapsPtr caps;
+static virDomainXMLConfPtr xmlconf;
 
 static virSecurityManagerPtr mgr;
 
@@ -165,7 +166,7 @@ testSELinuxLoadDef(const char *testname)
         goto cleanup;
     }
 
-    if (!(def = virDomainDefParseString(caps, xmlstr,
+    if (!(def = virDomainDefParseString(caps, xmlconf, xmlstr,
                                         QEMU_EXPECTED_VIRT_TYPES,
                                         0)))
         goto cleanup;
@@ -327,6 +328,9 @@ mymain(void)
     if ((caps = testQemuCapsInit()) == NULL)
         return EXIT_FAILURE;
 
+    if (!(xmlconf = virQEMUDriverCreateXMLConf()))
+        return EXIT_FAILURE;
+
 #define DO_TEST_LABELING(name) \
     if (virtTestRun("Labelling " # name, 1, testSELinuxLabeling, name) < 0) \
         ret = -1;                                                       \
