@@ -15,6 +15,12 @@ static int testXenDefaultConsoleType(const char *ostype,
         return VIR_DOMAIN_CHR_CONSOLE_TARGET_TYPE_XEN;
 }
 
+virDomainXMLConfPtr
+testXenXMLConfInit(void)
+{
+    return virDomainXMLConfNew(NULL, NULL);
+}
+
 virCapsPtr testXenCapsInit(void) {
     struct utsname utsname;
     virCapsPtr caps;
