@@ -89,7 +89,6 @@ static const vshCmdOptDef opts_connect[] = {
     },
     {.name = "readonly",
      .type = VSH_OT_BOOL,
-     .flags = 0,
      .help = N_("read-only connection")
     },
     {.name = NULL}
@@ -145,12 +144,10 @@ static const vshCmdInfo info_freecell[] = {
 static const vshCmdOptDef opts_freecell[] = {
     {.name = "cellno",
      .type = VSH_OT_INT,
-     .flags = 0,
      .help = N_("NUMA cell number")
     },
     {.name = "all",
      .type = VSH_OT_BOOL,
-     .flags = 0,
      .help = N_("show free memory for all NUMA cells")
     },
     {.name = NULL}
@@ -159,35 +156,29 @@ static const vshCmdOptDef opts_freecell[] = {
 static bool
 cmdFreecell(vshControl *ctl, const vshCmd *cmd)
 {
-    bool func_ret = false;
-    int ret;
-    int cell = -1, cell_given;
-    unsigned long long memory;
+    bool ret = false;
+    int cell = -1;
+    unsigned long long memory = 0;
     xmlNodePtr *nodes = NULL;
     unsigned long nodes_cnt;
     unsigned long *nodes_id = NULL;
     unsigned long long *nodes_free = NULL;
-    int all_given;
+    bool all = vshCommandOptBool(cmd, "all");
+    bool cellno = vshCommandOptBool(cmd, "cellno");
     int i;
     char *cap_xml = NULL;
     xmlDocPtr xml = NULL;
     xmlXPathContextPtr ctxt = NULL;
 
-    if ((cell_given = vshCommandOptInt(cmd, "cellno", &cell)) < 0) {
+    VSH_EXCLUSIVE_OPTIONS_VAR(all, cellno);
+
+    if (cellno && vshCommandOptInt(cmd, "cellno", &cell) < 0) {
         vshError(ctl, "%s", _("cell number has to be a number"));
-        goto cleanup;
-    }
-    all_given = vshCommandOptBool(cmd, "all");
-
-    if (all_given && cell_given) {
-        vshError(ctl, "%s", _("--cellno and --all are mutually exclusive. "
-                              "Please choose only one."));
-        goto cleanup;
+        return false;
     }
 
-    if (all_given) {
-        cap_xml = virConnectGetCapabilities(ctl->conn);
-        if (!cap_xml) {
+    if (all) {
+        if (!(cap_xml = virConnectGetCapabilities(ctl->conn))) {
             vshError(ctl, "%s", _("unable to get node capabilities"));
             goto cleanup;
         }
@@ -197,6 +188,7 @@ cmdFreecell(vshControl *ctl, const vshCmd *cmd)
             vshError(ctl, "%s", _("unable to get node capabilities"));
             goto cleanup;
         }
+
         nodes_cnt = virXPathNodeSet("/capabilities/host/topology/cells/cell",
                                     ctxt, &nodes);
 
@@ -219,15 +211,14 @@ cmdFreecell(vshControl *ctl, const vshCmd *cmd)
             }
             VIR_FREE(val);
             nodes_id[i]=id;
-            ret = virNodeGetCellsFreeMemory(ctl->conn, &(nodes_free[i]), id, 1);
-            if (ret != 1) {
+            if (virNodeGetCellsFreeMemory(ctl->conn, &(nodes_free[i]),
+                                          id, 1) != 1) {
                 vshError(ctl, _("failed to get free memory for NUMA node "
                                 "number: %lu"), id);
                 goto cleanup;
             }
         }
 
-        memory = 0;
         for (cell = 0; cell < nodes_cnt; cell++) {
             vshPrint(ctl, "%5lu: %10llu KiB\n", nodes_id[cell],
                     (nodes_free[cell]/1024));
@@ -237,23 +228,20 @@ cmdFreecell(vshControl *ctl, const vshCmd *cmd)
         vshPrintExtra(ctl, "--------------------\n");
         vshPrintExtra(ctl, "%5s: %10llu KiB\n", _("Total"), memory/1024);
     } else {
-        if (!cell_given) {
-            memory = virNodeGetFreeMemory(ctl->conn);
-            if (memory == 0)
+        if (cellno) {
+            if (virNodeGetCellsFreeMemory(ctl->conn, &memory, cell, 1) != 1)
                 goto cleanup;
-        } else {
-            ret = virNodeGetCellsFreeMemory(ctl->conn, &memory, cell, 1);
-            if (ret != 1)
-                goto cleanup;
-        }
 
-        if (cell == -1)
-            vshPrint(ctl, "%s: %llu KiB\n", _("Total"), (memory/1024));
-        else
             vshPrint(ctl, "%d: %llu KiB\n", cell, (memory/1024));
+        } else {
+            if ((memory = virNodeGetFreeMemory(ctl->conn)) == 0)
+                goto cleanup;
+
+            vshPrint(ctl, "%s: %llu KiB\n", _("Total"), (memory/1024));
+        }
     }
 
-    func_ret = true;
+    ret = true;
 
 cleanup:
     xmlXPathFreeContext(ctxt);
@@ -262,7 +250,7 @@ cleanup:
     VIR_FREE(nodes_free);
     VIR_FREE(nodes_id);
     VIR_FREE(cap_xml);
-    return func_ret;
+    return ret;
 }
 
 /*
@@ -358,12 +346,10 @@ static const vshCmdInfo info_nodecpustats[] = {
 static const vshCmdOptDef opts_node_cpustats[] = {
     {.name = "cpu",
      .type = VSH_OT_INT,
-     .flags = 0,
      .help = N_("prints specified cpu statistics only.")
     },
     {.name = "percent",
      .type = VSH_OT_BOOL,
-     .flags = 0,
      .help = N_("prints by percentage during 1 second.")
     },
     {.name = NULL}
@@ -495,7 +481,6 @@ static const vshCmdInfo info_nodememstats[] = {
 static const vshCmdOptDef opts_node_memstats[] = {
     {.name = "cell",
      .type = VSH_OT_INT,
-     .flags = 0,
      .help = N_("prints specified cell statistics only.")
     },
     {.name = NULL}
@@ -718,7 +703,6 @@ static const vshCmdInfo info_version[] = {
 static const vshCmdOptDef opts_version[] = {
     {.name = "daemon",
      .type = VSH_OT_BOOL,
-     .flags = VSH_OFLAG_NONE,
      .help = N_("report daemon version too")
     },
     {.name = NULL}
@@ -817,19 +801,16 @@ static const vshCmdInfo info_node_memory_tune[] = {
 static const vshCmdOptDef opts_node_memory_tune[] = {
     {.name = "shm-pages-to-scan",
      .type = VSH_OT_INT,
-     .flags = VSH_OFLAG_NONE,
      .help =  N_("number of pages to scan before the shared memory service "
                  "goes to sleep")
     },
     {.name = "shm-sleep-millisecs",
      .type = VSH_OT_INT,
-     .flags = VSH_OFLAG_NONE,
      .help =  N_("number of millisecs the shared memory service should "
                  "sleep before next scan")
     },
     {.name = "shm-merge-across-nodes",
      .type = VSH_OT_INT,
-     .flags = VSH_OFLAG_NONE,
      .help =  N_("Specifies if pages from different numa nodes can be merged")
     },
     {.name = NULL}
