@@ -117,7 +117,8 @@ cmdSecretDefine(vshControl *ctl, const vshCmd *cmd)
 
 cleanup:
     VIR_FREE(buffer);
-    virSecretFree(res);
+    if (res)
+        virSecretFree(res);
     return ret;
 }
 
@@ -487,22 +488,18 @@ static const vshCmdInfo info_secret_list[] = {
 static const vshCmdOptDef opts_secret_list[] = {
     {.name = "ephemeral",
      .type = VSH_OT_BOOL,
-     .flags = 0,
      .help = N_("list ephemeral secrets")
     },
     {.name = "no-ephemeral",
      .type = VSH_OT_BOOL,
-     .flags = 0,
      .help = N_("list non-ephemeral secrets")
     },
     {.name = "private",
      .type = VSH_OT_BOOL,
-     .flags = 0,
      .help = N_("list private secrets")
     },
     {.name = "no-private",
      .type = VSH_OT_BOOL,
-     .flags = 0,
      .help = N_("list non-private secrets")
     },
     {.name = NULL}
